@@ -3,10 +3,12 @@
  *
  *	Postgres-version-specific routines
  *
- *	Copyright (c) 2010-2011, PostgreSQL Global Development Group
+ *	Copyright (c) 2010-2013, PostgreSQL Global Development Group
  *	contrib/pg_upgrade/version_old_8_3.c
  */
 
+#include "postgres_fe.h"
+
 #include "pg_upgrade.h"
 
 #include "access/transam.h"
@@ -26,10 +28,9 @@ old_8_3_check_for_name_data_type_usage(ClusterInfo *cluster)
 	bool		found = false;
 	char		output_path[MAXPGPATH];
 
-	prep_status("Checking for invalid 'name' user columns");
+	prep_status("Checking for invalid \"name\" user columns");
 
-	snprintf(output_path, sizeof(output_path), "%s/tables_using_name.txt",
-			 os_info.cwd);
+	snprintf(output_path, sizeof(output_path), "tables_using_name.txt");
 
 	for (dbnum = 0; dbnum < cluster->dbarr.ndbs; dbnum++)
 	{
@@ -59,10 +60,10 @@ old_8_3_check_for_name_data_type_usage(ClusterInfo *cluster)
 								"		NOT a.attisdropped AND "
 								"		a.atttypid = 'pg_catalog.name'::pg_catalog.regtype AND "
 								"		c.relnamespace = n.oid AND "
-								 /* exclude possible orphaned temp tables */
+		/* exclude possible orphaned temp tables */
 								"  		n.nspname !~ '^pg_temp_' AND "
-								"		n.nspname !~ '^pg_toast_temp_' AND "
-						 		"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
+						 "		n.nspname !~ '^pg_toast_temp_' AND "
+								"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
 
 		ntups = PQntuples(res);
 		i_nspname = PQfnumber(res, "nspname");
@@ -71,11 +72,11 @@ old_8_3_check_for_name_data_type_usage(ClusterInfo *cluster)
 		for (rowno = 0; rowno < ntups; rowno++)
 		{
 			found = true;
-			if (script == NULL && (script = fopen(output_path, "w")) == NULL)
-				pg_log(PG_FATAL, "could not create necessary file:  %s\n", output_path);
+			if (script == NULL && (script = fopen_priv(output_path, "w")) == NULL)
+				pg_log(PG_FATAL, "could not open file \"%s\": %s\n", output_path, getErrorText(errno));
 			if (!db_used)
 			{
-				fprintf(script, "Database:  %s\n", active_db->db_name);
+				fprintf(script, "Database: %s\n", active_db->db_name);
 				db_used = true;
 			}
 			fprintf(script, "  %s.%s.%s\n",
@@ -96,13 +97,12 @@ old_8_3_check_for_name_data_type_usage(ClusterInfo *cluster)
 	{
 		pg_log(PG_REPORT, "fatal\n");
 		pg_log(PG_FATAL,
-			   "| Your installation contains the \"name\" data type in\n"
-			   "| user tables.  This data type changed its internal\n"
-			   "| alignment between your old and new clusters so this\n"
-			   "| cluster cannot currently be upgraded.  You can\n"
-			   "| remove the problem tables and restart the upgrade.\n"
-			   "| A list of the problem columns is in the file:\n"
-			   "| \t%s\n\n", output_path);
+			   "Your installation contains the \"name\" data type in user tables.  This\n"
+		"data type changed its internal alignment between your old and new\n"
+			   "clusters so this cluster cannot currently be upgraded.  You can remove\n"
+		"the problem tables and restart the upgrade.  A list of the problem\n"
+			   "columns is in the file:\n"
+			   "    %s\n\n", output_path);
 	}
 	else
 		check_ok();
@@ -125,8 +125,7 @@ old_8_3_check_for_tsquery_usage(ClusterInfo *cluster)
 
 	prep_status("Checking for tsquery user columns");
 
-	snprintf(output_path, sizeof(output_path), "%s/tables_using_tsquery.txt",
-			 os_info.cwd);
+	snprintf(output_path, sizeof(output_path), "tables_using_tsquery.txt");
 
 	for (dbnum = 0; dbnum < cluster->dbarr.ndbs; dbnum++)
 	{
@@ -146,15 +145,16 @@ old_8_3_check_for_tsquery_usage(ClusterInfo *cluster)
 								"FROM	pg_catalog.pg_class c, "
 								"		pg_catalog.pg_namespace n, "
 								"		pg_catalog.pg_attribute a "
+		/* materialized views didn't exist in 8.3, so no need to check 'm' */
 								"WHERE	c.relkind = 'r' AND "
 								"		c.oid = a.attrelid AND "
 								"		NOT a.attisdropped AND "
 								"		a.atttypid = 'pg_catalog.tsquery'::pg_catalog.regtype AND "
 								"		c.relnamespace = n.oid AND "
-								 /* exclude possible orphaned temp tables */
+		/* exclude possible orphaned temp tables */
 								"  		n.nspname !~ '^pg_temp_' AND "
-								"		n.nspname !~ '^pg_toast_temp_' AND "
-						 		"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
+						 "		n.nspname !~ '^pg_toast_temp_' AND "
+								"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
 
 		ntups = PQntuples(res);
 		i_nspname = PQfnumber(res, "nspname");
@@ -163,11 +163,11 @@ old_8_3_check_for_tsquery_usage(ClusterInfo *cluster)
 		for (rowno = 0; rowno < ntups; rowno++)
 		{
 			found = true;
-			if (script == NULL && (script = fopen(output_path, "w")) == NULL)
-				pg_log(PG_FATAL, "could not create necessary file:  %s\n", output_path);
+			if (script == NULL && (script = fopen_priv(output_path, "w")) == NULL)
+				pg_log(PG_FATAL, "could not open file \"%s\": %s\n", output_path, getErrorText(errno));
 			if (!db_used)
 			{
-				fprintf(script, "Database:  %s\n", active_db->db_name);
+				fprintf(script, "Database: %s\n", active_db->db_name);
 				db_used = true;
 			}
 			fprintf(script, "  %s.%s.%s\n",
@@ -188,13 +188,12 @@ old_8_3_check_for_tsquery_usage(ClusterInfo *cluster)
 	{
 		pg_log(PG_REPORT, "fatal\n");
 		pg_log(PG_FATAL,
-			   "| Your installation contains the \"tsquery\" data type.\n"
-			   "| This data type added a new internal field between\n"
-			   "| your old and new clusters so this cluster cannot\n"
-			   "| currently be upgraded.  You can remove the problem\n"
-			   "| columns and restart the upgrade.  A list of the\n"
-			   "| problem columns is in the file:\n"
-			   "| \t%s\n\n", output_path);
+			   "Your installation contains the \"tsquery\" data type.    This data type\n"
+			   "added a new internal field between your old and new clusters so this\n"
+		"cluster cannot currently be upgraded.  You can remove the problem\n"
+			   "columns and restart the upgrade.  A list of the problem columns is in the\n"
+			   "file:\n"
+			   "    %s\n\n", output_path);
 	}
 	else
 		check_ok();
@@ -216,8 +215,7 @@ old_8_3_check_ltree_usage(ClusterInfo *cluster)
 
 	prep_status("Checking for contrib/ltree");
 
-	snprintf(output_path, sizeof(output_path), "%s/contrib_ltree.txt",
-			 os_info.cwd);
+	snprintf(output_path, sizeof(output_path), "contrib_ltree.txt");
 
 	for (dbnum = 0; dbnum < cluster->dbarr.ndbs; dbnum++)
 	{
@@ -244,7 +242,7 @@ old_8_3_check_ltree_usage(ClusterInfo *cluster)
 		for (rowno = 0; rowno < ntups; rowno++)
 		{
 			found = true;
-			if (script == NULL && (script = fopen(output_path, "w")) == NULL)
+			if (script == NULL && (script = fopen_priv(output_path, "w")) == NULL)
 				pg_log(PG_FATAL, "Could not open file \"%s\": %s\n",
 					   output_path, getErrorText(errno));
 			if (!db_used)
@@ -304,8 +302,7 @@ old_8_3_rebuild_tsvector_tables(ClusterInfo *cluster, bool check_mode)
 
 	prep_status("Checking for tsvector user columns");
 
-	snprintf(output_path, sizeof(output_path), "%s/rebuild_tsvector_tables.sql",
-			 os_info.cwd);
+	snprintf(output_path, sizeof(output_path), "rebuild_tsvector_tables.sql");
 
 	for (dbnum = 0; dbnum < cluster->dbarr.ndbs; dbnum++)
 	{
@@ -327,15 +324,16 @@ old_8_3_rebuild_tsvector_tables(ClusterInfo *cluster, bool check_mode)
 								"FROM	pg_catalog.pg_class c, "
 								"		pg_catalog.pg_namespace n, "
 								"		pg_catalog.pg_attribute a "
+		/* materialized views didn't exist in 8.3, so no need to check 'm' */
 								"WHERE	c.relkind = 'r' AND "
 								"		c.oid = a.attrelid AND "
 								"		NOT a.attisdropped AND "
 								"		a.atttypid = 'pg_catalog.tsvector'::pg_catalog.regtype AND "
 								"		c.relnamespace = n.oid AND "
-								 /* exclude possible orphaned temp tables */
+		/* exclude possible orphaned temp tables */
 								"  		n.nspname !~ '^pg_temp_' AND "
-								"		n.nspname !~ '^pg_toast_temp_' AND "
-						 		"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
+						 "		n.nspname !~ '^pg_toast_temp_' AND "
+								"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
 
 /*
  *	This macro is used below to avoid reindexing indexes already rebuilt
@@ -347,6 +345,7 @@ old_8_3_rebuild_tsvector_tables(ClusterInfo *cluster, bool check_mode)
 								"FROM	pg_catalog.pg_class c, "		\
 								"		pg_catalog.pg_namespace n, "	\
 								"		pg_catalog.pg_attribute a "		\
+		/* materialized views didn't exist in 8.3, so no need to check 'm' */ \
 								"WHERE	c.relkind = 'r' AND "			\
 								"		c.oid = a.attrelid AND "		\
 								"		NOT a.attisdropped AND "		\
@@ -364,17 +363,12 @@ old_8_3_rebuild_tsvector_tables(ClusterInfo *cluster, bool check_mode)
 			found = true;
 			if (!check_mode)
 			{
-				if (script == NULL && (script = fopen(output_path, "w")) == NULL)
-					pg_log(PG_FATAL, "could not create necessary file:  %s\n", output_path);
+				if (script == NULL && (script = fopen_priv(output_path, "w")) == NULL)
+					pg_log(PG_FATAL, "could not open file \"%s\": %s\n", output_path, getErrorText(errno));
 				if (!db_used)
 				{
-					PQExpBufferData connectbuf;
-
-					initPQExpBuffer(&connectbuf);
-					appendPsqlMetaConnect(&connectbuf, active_db->db_name);
-					appendPQExpBufferChar(&connectbuf, '\n');
-					fputs(connectbuf.data, script);
-					termPQExpBuffer(&connectbuf);
+					fprintf(script, "\\connect %s\n\n",
+							quote_identifier(active_db->db_name));
 					db_used = true;
 				}
 
@@ -405,7 +399,7 @@ old_8_3_rebuild_tsvector_tables(ClusterInfo *cluster, bool check_mode)
 
 		PQclear(res);
 
-		/* XXX Mark tables as not accessable somehow */
+		/* XXX Mark tables as not accessible somehow */
 
 		PQfinish(conn);
 	}
@@ -418,20 +412,17 @@ old_8_3_rebuild_tsvector_tables(ClusterInfo *cluster, bool check_mode)
 		report_status(PG_WARNING, "warning");
 		if (check_mode)
 			pg_log(PG_WARNING, "\n"
-				   "| Your installation contains tsvector columns.\n"
-				   "| The tsvector internal storage format changed\n"
-				   "| between your old and new clusters so the tables\n"
-				   "| must be rebuilt.  After upgrading, you will be\n"
-				   "| given instructions.\n\n");
+				   "Your installation contains tsvector columns.  The tsvector internal\n"
+				   "storage format changed between your old and new clusters so the tables\n"
+				   "must be rebuilt.  After upgrading, you will be given instructions.\n\n");
 		else
 			pg_log(PG_WARNING, "\n"
-				   "| Your installation contains tsvector columns.\n"
-				   "| The tsvector internal storage format changed\n"
-				   "| between your old and new clusters so the tables\n"
-				   "| must be rebuilt.  The file:\n"
-				   "| \t%s\n"
-				   "| when executed by psql by the database super-user\n"
-				   "| will rebuild all tables with tsvector columns.\n\n",
+				   "Your installation contains tsvector columns.  The tsvector internal\n"
+				   "storage format changed between your old and new clusters so the tables\n"
+				   "must be rebuilt.  The file:\n"
+				   "    %s\n"
+				   "when executed by psql by the database superuser will rebuild all tables\n"
+				   "with tsvector columns.\n\n",
 				   output_path);
 	}
 	else
@@ -452,10 +443,9 @@ old_8_3_invalidate_hash_gin_indexes(ClusterInfo *cluster, bool check_mode)
 	bool		found = false;
 	char		output_path[MAXPGPATH];
 
-	prep_status("Checking for hash and gin indexes");
+	prep_status("Checking for hash and GIN indexes");
 
-	snprintf(output_path, sizeof(output_path), "%s/reindex_hash_and_gin.sql",
-			 os_info.cwd);
+	snprintf(output_path, sizeof(output_path), "reindex_hash_and_gin.sql");
 
 	for (dbnum = 0; dbnum < cluster->dbarr.ndbs; dbnum++)
 	{
@@ -471,11 +461,11 @@ old_8_3_invalidate_hash_gin_indexes(ClusterInfo *cluster, bool check_mode)
 		/* find hash and gin indexes */
 		res = executeQueryOrDie(conn,
 								"SELECT n.nspname, c.relname "
-								"FROM 	pg_catalog.pg_class c, "
+								"FROM	pg_catalog.pg_class c, "
 								"		pg_catalog.pg_index i, "
 								"		pg_catalog.pg_am a, "
 								"		pg_catalog.pg_namespace n "
-								"WHERE 	i.indexrelid = c.oid AND "
+								"WHERE	i.indexrelid = c.oid AND "
 								"		c.relam = a.oid AND "
 								"		c.relnamespace = n.oid AND "
 							"		a.amname IN ('hash', 'gin') AND "
@@ -489,16 +479,12 @@ old_8_3_invalidate_hash_gin_indexes(ClusterInfo *cluster, bool check_mode)
 			found = true;
 			if (!check_mode)
 			{
-				if (script == NULL && (script = fopen(output_path, "w")) == NULL)
-					pg_log(PG_FATAL, "could not create necessary file:  %s\n", output_path);
+				if (script == NULL && (script = fopen_priv(output_path, "w")) == NULL)
+					pg_log(PG_FATAL, "could not open file \"%s\": %s\n", output_path, getErrorText(errno));
 				if (!db_used)
 				{
-					PQExpBufferData connectbuf;
-
-					initPQExpBuffer(&connectbuf);
-					appendPsqlMetaConnect(&connectbuf, active_db->db_name);
-					fputs(connectbuf.data, script);
-					termPQExpBuffer(&connectbuf);
+					fprintf(script, "\\connect %s\n",
+							quote_identifier(active_db->db_name));
 					db_used = true;
 				}
 				fprintf(script, "REINDEX INDEX %s.%s;\n",
@@ -514,10 +500,10 @@ old_8_3_invalidate_hash_gin_indexes(ClusterInfo *cluster, bool check_mode)
 			PQclear(executeQueryOrDie(conn,
 									  "UPDATE pg_catalog.pg_index i "
 									  "SET	indisvalid = false "
-									  "FROM 	pg_catalog.pg_class c, "
+									  "FROM	pg_catalog.pg_class c, "
 									  "		pg_catalog.pg_am a, "
 									  "		pg_catalog.pg_namespace n "
-									  "WHERE 	i.indexrelid = c.oid AND "
+									  "WHERE	i.indexrelid = c.oid AND "
 									  "		c.relam = a.oid AND "
 									  "		c.relnamespace = n.oid AND "
 									"		a.amname IN ('hash', 'gin')"));
@@ -533,23 +519,18 @@ old_8_3_invalidate_hash_gin_indexes(ClusterInfo *cluster, bool check_mode)
 		report_status(PG_WARNING, "warning");
 		if (check_mode)
 			pg_log(PG_WARNING, "\n"
-				   "| Your installation contains hash and/or gin\n"
-				   "| indexes.  These indexes have different\n"
-				   "| internal formats between your old and new\n"
-				   "| clusters so they must be reindexed with the\n"
-				   "| REINDEX command. After upgrading, you will\n"
-				   "| be given REINDEX instructions.\n\n");
+				   "Your installation contains hash and/or GIN indexes.  These indexes have\n"
+				   "different internal formats between your old and new clusters, so they\n"
+				   "must be reindexed with the REINDEX command.  After upgrading, you will\n"
+				   "be given REINDEX instructions.\n\n");
 		else
 			pg_log(PG_WARNING, "\n"
-				   "| Your installation contains hash and/or gin\n"
-				   "| indexes.  These indexes have different internal\n"
-				   "| formats between your old and new clusters so\n"
-				   "| they must be reindexed with the REINDEX command.\n"
-				   "| The file:\n"
-				   "| \t%s\n"
-				   "| when executed by psql by the database super-user\n"
-				   "| will recreate all invalid indexes; until then,\n"
-				   "| none of these indexes will be used.\n\n",
+				   "Your installation contains hash and/or GIN indexes.  These indexes have\n"
+				   "different internal formats between your old and new clusters, so they\n"
+				   "must be reindexed with the REINDEX command.  The file:\n"
+				   "    %s\n"
+				   "when executed by psql by the database superuser will recreate all invalid\n"
+			  "indexes; until then, none of these indexes will be used.\n\n",
 				   output_path);
 	}
 	else
@@ -573,8 +554,7 @@ old_8_3_invalidate_bpchar_pattern_ops_indexes(ClusterInfo *cluster,
 
 	prep_status("Checking for bpchar_pattern_ops indexes");
 
-	snprintf(output_path, sizeof(output_path), "%s/reindex_bpchar_ops.sql",
-			 os_info.cwd);
+	snprintf(output_path, sizeof(output_path), "reindex_bpchar_ops.sql");
 
 	for (dbnum = 0; dbnum < cluster->dbarr.ndbs; dbnum++)
 	{
@@ -618,16 +598,12 @@ old_8_3_invalidate_bpchar_pattern_ops_indexes(ClusterInfo *cluster,
 			found = true;
 			if (!check_mode)
 			{
-				if (script == NULL && (script = fopen(output_path, "w")) == NULL)
-					pg_log(PG_FATAL, "could not create necessary file:  %s\n", output_path);
+				if (script == NULL && (script = fopen_priv(output_path, "w")) == NULL)
+					pg_log(PG_FATAL, "could not open file \"%s\": %s\n", output_path, getErrorText(errno));
 				if (!db_used)
 				{
-					PQExpBufferData connectbuf;
-
-					initPQExpBuffer(&connectbuf);
-					appendPsqlMetaConnect(&connectbuf, active_db->db_name);
-					fputs(connectbuf.data, script);
-					termPQExpBuffer(&connectbuf);
+					fprintf(script, "\\connect %s\n",
+							quote_identifier(active_db->db_name));
 					db_used = true;
 				}
 				fprintf(script, "REINDEX INDEX %s.%s;\n",
@@ -667,23 +643,18 @@ old_8_3_invalidate_bpchar_pattern_ops_indexes(ClusterInfo *cluster,
 		report_status(PG_WARNING, "warning");
 		if (check_mode)
 			pg_log(PG_WARNING, "\n"
-				   "| Your installation contains indexes using\n"
-				   "| \"bpchar_pattern_ops\".  These indexes have\n"
-				   "| different internal formats between your old and\n"
-				   "| new clusters so they must be reindexed with the\n"
-				   "| REINDEX command.  After upgrading, you will be\n"
-				   "| given REINDEX instructions.\n\n");
+				   "Your installation contains indexes using \"bpchar_pattern_ops\".  These\n"
+				   "indexes have different internal formats between your old and new clusters\n"
+				   "so they must be reindexed with the REINDEX command.  After upgrading, you\n"
+				   "will be given REINDEX instructions.\n\n");
 		else
 			pg_log(PG_WARNING, "\n"
-				   "| Your installation contains indexes using\n"
-				   "| \"bpchar_pattern_ops\".  These indexes have\n"
-				   "| different internal formats between your old and\n"
-				   "| new clusters so they must be reindexed with the\n"
-				   "| REINDEX command.  The file:\n"
-				   "| \t%s\n"
-				   "| when executed by psql by the database super-user\n"
-				   "| will recreate all invalid indexes; until then,\n"
-				   "| none of these indexes will be used.\n\n",
+				   "Your installation contains indexes using \"bpchar_pattern_ops\".  These\n"
+				   "indexes have different internal formats between your old and new clusters\n"
+			"so they must be reindexed with the REINDEX command.  The file:\n"
+				   "    %s\n"
+				   "when executed by psql by the database superuser will recreate all invalid\n"
+			  "indexes; until then, none of these indexes will be used.\n\n",
 				   output_path);
 	}
 	else
@@ -698,7 +669,7 @@ old_8_3_invalidate_bpchar_pattern_ops_indexes(ClusterInfo *cluster,
  *	we don't transfer sequence files but instead use the CREATE SEQUENCE
  *	command from the schema dump, and use setval() to restore the sequence
  *	value and 'is_called' from the old database.  This is safe to run
- *	by pg_upgrade because sequence files are not transfered from the old
+ *	by pg_upgrade because sequence files are not transferred from the old
  *	server, even in link mode.
  */
 char *
@@ -709,7 +680,7 @@ old_8_3_create_sequence_script(ClusterInfo *cluster)
 	bool		found = false;
 	char	   *output_path = pg_malloc(MAXPGPATH);
 
-	snprintf(output_path, MAXPGPATH, "%s/adjust_sequences.sql", os_info.cwd);
+	snprintf(output_path, MAXPGPATH, "adjust_sequences.sql");
 
 	prep_status("Creating script to adjust sequences");
 
@@ -731,10 +702,10 @@ old_8_3_create_sequence_script(ClusterInfo *cluster)
 								"		pg_catalog.pg_namespace n "
 								"WHERE	c.relkind = 'S' AND "
 								"		c.relnamespace = n.oid AND "
-								 /* exclude possible orphaned temp tables */
+		/* exclude possible orphaned temp tables */
 								"  		n.nspname !~ '^pg_temp_' AND "
-								"		n.nspname !~ '^pg_toast_temp_' AND "
-						 		"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
+						 "		n.nspname !~ '^pg_toast_temp_' AND "
+								"		n.nspname NOT IN ('pg_catalog', 'information_schema')");
 
 		ntups = PQntuples(res);
 		i_nspname = PQfnumber(res, "nspname");
@@ -749,17 +720,12 @@ old_8_3_create_sequence_script(ClusterInfo *cluster)
 
 			found = true;
 
-			if (script == NULL && (script = fopen(output_path, "w")) == NULL)
-				pg_log(PG_FATAL, "could not create necessary file:  %s\n", output_path);
+			if (script == NULL && (script = fopen_priv(output_path, "w")) == NULL)
+				pg_log(PG_FATAL, "could not open file \"%s\": %s\n", output_path, getErrorText(errno));
 			if (!db_used)
 			{
-				PQExpBufferData connectbuf;
-
-				initPQExpBuffer(&connectbuf);
-				appendPsqlMetaConnect(&connectbuf, active_db->db_name);
-				appendPQExpBufferChar(&connectbuf, '\n');
-				fputs(connectbuf.data, script);
-				termPQExpBuffer(&connectbuf);
+				fprintf(script, "\\connect %s\n\n",
+						quote_identifier(active_db->db_name));
 				db_used = true;
 			}
 
