@@ -54,13 +54,20 @@ extern int pg_next_dst_boundary(const pg_time_t *timep,
 					 long int *after_gmtoff,
 					 int *after_isdst,
 					 const pg_tz *tz);
-extern size_t pg_strftime(char *s, size_t max, const char *format,
-			const struct pg_tm * tm);
-
+extern bool pg_interpret_timezone_abbrev(const char *abbrev,
+							 const pg_time_t *timep,
+							 long int *gmtoff,
+							 int *isdst,
+							 const pg_tz *tz);
 extern bool pg_get_timezone_offset(const pg_tz *tz, long int *gmtoff);
 extern const char *pg_get_timezone_name(pg_tz *tz);
 extern bool pg_tz_acceptable(pg_tz *tz);
 
+/* these functions are in strftime.c */
+
+extern size_t pg_strftime(char *s, size_t max, const char *format,
+			const struct pg_tm * tm);
+
 /* these functions and variables are in pgtz.c */
 
 extern pg_tz *session_timezone;
