@@ -37,13 +37,15 @@ sub Install
 	$| = 1;
 
 	my $target = shift;
+
 	# if called from vcregress, the config will be passed to us
 	# so no need to re-include these
 	our $config = shift;
 	unless ($config)
 	{
+
 		# suppress warning about harmless redeclaration of $config
-		no warnings 'misc'; 
+		no warnings 'misc';
 		require "config_default.pl";
 		require "config.pl" if (-f "config.pl");
 	}
@@ -83,10 +85,15 @@ sub Install
 		"src");
 	CopySetOfFiles('config files', $sample_files, $target . '/share/');
 	CopyFiles(
-		'Import libraries',         $target . '/lib/',
-		"$conf\\",                  "postgres\\postgres.lib",
-		"libpq\\libpq.lib",         "libecpg\\libecpg.lib",
-		"libpgport\\libpgport.lib", "libpgtypes\\libpgtypes.lib",
+		'Import libraries',
+		$target . '/lib/',
+		"$conf\\",
+		"postgres\\postgres.lib",
+		"libpq\\libpq.lib",
+		"libecpg\\libecpg.lib",
+		"libpgcommon\\libpgcommon.lib",
+		"libpgport\\libpgport.lib",
+		"libpgtypes\\libpgtypes.lib",
 		"libecpg_compat\\libecpg_compat.lib");
 	CopySetOfFiles(
 		'timezone names',
@@ -309,21 +316,12 @@ sub GenerateTimezoneFiles
 	my $mf     = read_file("src/timezone/Makefile");
 	$mf =~ s{\\\s*[\r\n]+}{}mg;
 	$mf =~ /^TZDATA\s*:?=\s*(.*)$/m
-	  || die "Could not find TZDATA line in timezone makefile\n";
+	  || die "Could not find TZDATA row in timezone makefile\n";
 	my @tzfiles = split /\s+/, $1;
-
+	unshift @tzfiles, '';
 	print "Generating timezone files...";
-
-	my @args = ("$conf/zic/zic",
-				'-d',
-				"$target/share/timezone");
-	foreach (@tzfiles)
-	{
-		my $tzfile = $_;
-		push(@args, "src/timezone/data/$tzfile")
-	}
-
-	system(@args);
+	system("$conf\\zic\\zic -d \"$target/share/timezone\" "
+		  . join(" src/timezone/data/", @tzfiles));
 	print "\n";
 }
 
@@ -498,10 +496,10 @@ sub CopyIncludeFiles
 		'include/internal/libpq', 'include/server', 'include/server/parser');
 
 	CopyFiles(
-		'Public headers',
-		$target . '/include/',
-		'src/include/', 'postgres_ext.h', 'pg_config.h', 'pg_config_os.h',
-		'dynloader.h', 'pg_config_manual.h');
+		'Public headers', $target . '/include/',
+		'src/include/',   'postgres_ext.h',
+		'pg_config.h',    'pg_config_ext.h',
+		'pg_config_os.h', 'dynloader.h', 'pg_config_manual.h');
 	lcopy('src/include/libpq/libpq-fs.h', $target . '/include/libpq/')
 	  || croak 'Could not copy libpq-fs.h';
 
@@ -524,7 +522,8 @@ sub CopyIncludeFiles
 	CopyFiles(
 		'Server headers',
 		$target . '/include/server/',
-		'src/include/', 'pg_config.h', 'pg_config_os.h', 'dynloader.h');
+		'src/include/', 'pg_config.h', 'pg_config_ext.h', 'pg_config_os.h',
+		'dynloader.h');
 	CopyFiles(
 		'Grammar header',
 		$target . '/include/server/parser/',
@@ -551,10 +550,9 @@ sub CopyIncludeFiles
 		next unless (-d "src/include/$d");
 
 		EnsureDirectories("$target/include/server/$d");
-		my @args = ('xcopy', '/s', '/i', '/q', '/r', '/y',
-				 "src\\include\\$d\\*.h",
-				 "$ctarget\\include\\server\\$d\\");
-		system(@args) && croak("Failed to copy include directory $d\n");
+		system(
+qq{xcopy /s /i /q /r /y src\\include\\$d\\*.h "$ctarget\\include\\server\\$d\\"}
+		) && croak("Failed to copy include directory $d\n");
 	}
 	closedir($D);
 
@@ -607,11 +605,9 @@ sub GenerateNLSFiles
 
 			EnsureDirectories($target, "share/locale/$lang",
 				"share/locale/$lang/LC_MESSAGES");
-			my @args = ("$nlspath\\bin\\msgfmt",
-			   '-o',
-			   "$target\\share\\locale\\$lang\\LC_MESSAGES\\$prgm-$majorver.mo",
-			   $_);
-			system(@args) && croak("Could not run msgfmt on $dir\\$_");
+			system(
+"\"$nlspath\\bin\\msgfmt\" -o \"$target\\share\\locale\\$lang\\LC_MESSAGES\\$prgm-$majorver.mo\" $_"
+			) && croak("Could not run msgfmt on $dir\\$_");
 			print ".";
 		}
 	}
